<?php
/*********************/
/*                   */
/*  Version : 5.1.0  */
/*  Author  : RM     */
/*  Comment : 071223 */
/*                   */
/*********************/

class OAuthException
{

}

class OAuthConsumer
{

    public $key = NULL;
    public $secret = NULL;

    public function __construct( $key, $secret, $callback_url = NULL )
    {
        $this->key = $key;
        $this->secret = $secret;
        $this->callback_url = $callback_url;
    }

    public function __toString( )
    {
        return "OAuthConsumer[key={$this->key},secret={$this->secret}]";
    }

}

class OAuthToken
{

    public $key = NULL;
    public $secret = NULL;

    public function __construct( $key, $secret )
    {
        $this->key = $key;
        $this->secret = $secret;
    }

    public function to_string( )
    {
        return "oauth_token=".OAuthUtil::urlencode_rfc3986( $this->key )."&oauth_token_secret=".OAuthUtil::urlencode_rfc3986( $this->secret );
    }

    public function __toString( )
    {
        return $this->to_string( );
    }

}

class OAuthSignatureMethod
{

    public function check_signature( &$request, $consumer, $token, $signature )
    {
        $built = $this->build_signature( $request, $consumer, $token );
        return $built == $signature;
    }

}

class OAuthSignatureMethod_HMAC_SHA1 extends OAuthSignatureMethod
{

    public function get_name( )
    {
        return "HMAC-SHA1";
    }

    public function build_signature( $request, $consumer, $token )
    {
        $base_string = $request->get_signature_base_string( );
        $request->base_string = $base_string;
        $key_parts = array(
            $consumer->secret,
            $token ? $token->secret : ""
        );
        $key_parts = OAuthUtil::urlencode_rfc3986( $key_parts );
        $key = implode( "&", $key_parts );
        return base64_encode( hash_hmac( "sha1", $base_string, $key, true ) );
    }

}

class OAuthSignatureMethod_PLAINTEXT extends OAuthSignatureMethod
{

    public function get_name( )
    {
        return "PLAINTEXT";
    }

    public function build_signature( $request, $consumer, $token )
    {
        $sig = array(
            OAuthUtil::urlencode_rfc3986( $consumer->secret )
        );
        if ( $token )
        {
            array_push( $sig, OAuthUtil::urlencode_rfc3986( $token->secret ) );
        }
        else
        {
            array_push( $sig, "" );
        }
        $raw = implode( "&", $sig );
        $request->base_string = $raw;
        return OAuthUtil::urlencode_rfc3986( $raw );
    }

}

class OAuthSignatureMethod_RSA_SHA1 extends OAuthSignatureMethod
{

    public function get_name( )
    {
        return "RSA-SHA1";
    }

    protected function fetch_public_cert( &$request )
    {
        throw Exception( "fetch_public_cert not implemented" );
    }

    protected function fetch_private_cert( &$request )
    {
        throw Exception( "fetch_private_cert not implemented" );
    }

    public function build_signature( &$request, $consumer, $token )
    {
        $base_string = $request->get_signature_base_string( );
        $request->base_string = $base_string;
        $cert = $this->fetch_private_cert( $request );
        $privatekeyid = openssl_get_privatekey( $cert );
        $ok = openssl_sign( $base_string, $signature, $privatekeyid );
        openssl_free_key( $privatekeyid );
        return base64_encode( $signature );
    }

    public function check_signature( &$request, $consumer, $token, $signature )
    {
        $decoded_sig = base64_decode( $signature );
        $base_string = $request->get_signature_base_string( );
        $cert = $this->fetch_public_cert( $request );
        $publickeyid = openssl_get_publickey( $cert );
        $ok = openssl_verify( $base_string, $decoded_sig, $publickeyid );
        openssl_free_key( $publickeyid );
        return $ok == 1;
    }

}

class OAuthRequest
{

    private $parameters = NULL;
    private $http_method = NULL;
    private $http_url = NULL;
    public $base_string = NULL;
    public static $version = "1.0";

    public function __construct( $http_method, $http_url, $parameters = NULL )
    {
        if ( !$parameters )
        {
            $parameters = array( );
        }
        $this->parameters = $parameters;
        $this->http_method = $http_method;
        $this->http_url = $http_url;
    }

    public static function from_request( $http_method = NULL, $http_url = NULL, $parameters = NULL )
    {
        $scheme = !isset( $_SERVER['HTTPS'] ) || $_SERVER['HTTPS'] != "on" ? "http" : "https";
        if ( !$http_url )
        {
            $http_url = $scheme."://".$_SERVER['HTTP_HOST'].":".$_SERVER['SERVER_PORT'].$_SERVER['REQUEST_URI'];
        }
        if ( !$http_method )
        {
            $http_method = $_SERVER['REQUEST_METHOD'];
        }
        $request_headers = OAuthRequest::get_headers( );
        if ( $parameters )
        {
            $req = new OAuthRequest( $http_method, $http_url, $parameters );
        }
        else
        {
            $req_parameters = $_GET;
            if ( $http_method == "POST" && @strstr( $request_headers['Content-Type'], "application/x-www-form-urlencoded" ) )
            {
                $req_parameters = array_merge( $req_parameters, $_POST );
            }
            if ( substr( $request_headers['Authorization'], 0, 6 ) == "OAuth " )
            {
                $header_parameters = OAuthRequest::split_header( $request_headers['Authorization'] );
                $parameters = array_merge( $req_parameters, $header_parameters );
                $req = new OAuthRequest( $http_method, $http_url, $parameters );
            }
            else
            {
                $req = new OAuthRequest( $http_method, $http_url, $req_parameters );
            }
        }
        return $req;
    }

    public static function from_consumer_and_token( $consumer, $token, $http_method, $http_url, $parameters = NULL )
    {
        if ( !$parameters )
        {
            $parameters = array( );
        }
        $defaults = array(
            "oauth_version" => OAuthRequest::$version,
            "oauth_nonce" => OAuthRequest::generate_nonce( ),
            "oauth_timestamp" => OAuthRequest::generate_timestamp( ),
            "oauth_consumer_key" => $consumer->key
        );
        $parameters = array_merge( $defaults, $parameters );
        if ( $token )
        {
            $parameters['oauth_token'] = $token->key;
        }
        return new OAuthRequest( $http_method, $http_url, $parameters );
    }

    public function set_parameter( $name, $value )
    {
        $this->parameters[$name] = $value;
    }

    public function get_parameter( $name )
    {
        return isset( $this->parameters[$name] ) ? $this->parameters[$name] : null;
    }

    public function get_parameters( )
    {
        return $this->parameters;
    }

    public function get_signable_parameters( )
    {
        $params = $this->parameters;
        if ( isset( $params['oauth_signature'] ) )
        {
            unset( $params['oauth_signature'] );
        }
        $keys = OAuthUtil::urlencode_rfc3986( array_keys( $params ) );
        $values = OAuthUtil::urlencode_rfc3986( array_values( $params ) );
        $params = array_combine( $keys, $values );
        uksort( $params, "strcmp" );
        $pairs = array( );
        foreach ( $params as $key => $value )
        {
            if ( is_array( $value ) )
            {
                natsort( $value );
                foreach ( $value as $v2 )
                {
                    $pairs[] = $key."=".$v2;
                }
            }
            else
            {
                $pairs[] = $key."=".$value;
            }
        }
        return implode( "&", $pairs );
    }

    public function get_signature_base_string( )
    {
        $parts = array(
            $this->get_normalized_http_method( ),
            $this->get_normalized_http_url( ),
            $this->get_signable_parameters( )
        );
        $parts = OAuthUtil::urlencode_rfc3986( $parts );
        return implode( "&", $parts );
    }

    public function get_normalized_http_method( )
    {
        return strtoupper( $this->http_method );
    }

    public function get_normalized_http_url( )
    {
        $parts = parse_url( $this->http_url );
        $port = $parts['port'];
        $scheme = $parts['scheme'];
        $host = $parts['host'];
        $path = $parts['path'];
        if ( !$port )
        {
            $port = $scheme == "https" ? "443" : "80";
        }
        if ( $scheme == "https" && $port != "443" || $scheme == "http" && $port != "80" )
        {
            $host = "{$host}:{$port}";
        }
        return "{$scheme}://{$host}{$path}";
    }

    public function to_url( )
    {
        $out = $this->get_normalized_http_url( )."?";
        $out .= $this->to_postdata( );
        return $out;
    }

    public function to_postdata( )
    {
        $total = array( );
        foreach ( $this->parameters as $k => $v )
        {
            if ( is_array( $v ) )
            {
                foreach ( $v as $va )
                {
                    $total[] = OAuthUtil::urlencode_rfc3986( $k )."[]=".OAuthUtil::urlencode_rfc3986( $va );
                }
            }
            else
            {
                $total[] = OAuthUtil::urlencode_rfc3986( $k )."=".OAuthUtil::urlencode_rfc3986( $v );
            }
        }
        $out = implode( "&", $total );
        return $out;
    }

    public function to_header( )
    {
        $out = "Authorization: OAuth realm=\"\"";
        $total = array( );
        foreach ( $this->parameters as $k => $v )
        {
            if ( substr( $k, 0, 5 ) != "oauth" )
            {
                continue;
            }
            if ( is_array( $v ) )
            {
                throw new OAuthException( "Arrays not supported in headers" );
            }
            $out .= ",".OAuthUtil::urlencode_rfc3986( $k )."=\"".OAuthUtil::urlencode_rfc3986( $v )."\"";
        }
        return $out;
    }

    public function __toString( )
    {
        return $this->to_url( );
    }

    public function sign_request( $signature_method, $consumer, $token )
    {
        $this->set_parameter( "oauth_signature_method", $signature_method->get_name( ) );
        $signature = $this->build_signature( $signature_method, $consumer, $token );
        $this->set_parameter( "oauth_signature", $signature );
    }

    public function build_signature( $signature_method, $consumer, $token )
    {
        $signature = $signature_method->build_signature( $this, $consumer, $token );
        return $signature;
    }

    private static function generate_timestamp( )
    {
        return time( );
    }

    private static function generate_nonce( )
    {
        $mt = microtime( );
        $rand = mt_rand( );
        return md5( $mt.$rand );
    }

    private static function split_header( $header )
    {
        $pattern = "/(([-_a-z]*)=(\"([^\"]*)\"|([^,]*)),?)/";
        $offset = 0;
        $params = array( );
        while ( 0 < preg_match( $pattern, $header, $matches, PREG_OFFSET_CAPTURE, $offset ) )
        {
            $match = $matches[0];
            $header_name = $matches[2][0];
            $header_content = isset( $matches[5] ) ? $matches[5][0] : $matches[4][0];
            $params[$header_name] = OAuthUtil::urldecode_rfc3986( $header_content );
            $offset = $match[1] + strlen( $match[0] );
        }
        if ( isset( $params['realm'] ) )
        {
            unset( $params['realm'] );
        }
        return $params;
    }

    private static function get_headers( )
    {
        if ( function_exists( "apache_request_headers" ) )
        {
            return apache_request_headers( );
        }
        $out = array( );
        foreach ( $_SERVER as $key => $value )
        {
            if ( substr( $key, 0, 5 ) == "HTTP_" )
            {
                $key = str_replace( " ", "-", ucwords( strtolower( str_replace( "_", " ", substr( $key, 5 ) ) ) ) );
                $out[$key] = $value;
            }
        }
        return $out;
    }

}

class OAuthServer
{

    protected $timestamp_threshold = 300;
    protected $version = 1;
    protected $signature_methods = array( );
    protected $data_store = NULL;

    public function __construct( $data_store )
    {
        $this->data_store = $data_store;
    }

    public function add_signature_method( $signature_method )
    {
        $this->signature_methods[$signature_method->get_name( )] = $signature_method;
    }

    public function fetch_request_token( &$request )
    {
        $this->get_version( $request );
        $consumer = $this->get_consumer( $request );
        $token = NULL;
        $this->check_signature( $request, $consumer, $token );
        $new_token = $this->data_store->new_request_token( $consumer );
        return $new_token;
    }

    public function fetch_access_token( &$request )
    {
        $this->get_version( $request );
        $consumer = $this->get_consumer( $request );
        $token = $this->get_token( $request, $consumer, "request" );
        $this->check_signature( $request, $consumer, $token );
        $new_token = $this->data_store->new_access_token( $token, $consumer );
        return $new_token;
    }

    public function verify_request( &$request )
    {
        $this->get_version( $request );
        $consumer = $this->get_consumer( $request );
        $token = $this->get_token( $request, $consumer, "access" );
        $this->check_signature( $request, $consumer, $token );
        return array(
            $consumer,
            $token
        );
    }

    private function get_version( &$request )
    {
        $version = $request->get_parameter( "oauth_version" );
        if ( !$version )
        {
            $version = 1;
        }
        if ( $version && $version != $this->version )
        {
            throw new OAuthException( "OAuth version '{$version}' not supported" );
        }
        return $version;
    }

    private function get_signature_method( &$request )
    {
        $signature_method = @$request->get_parameter( "oauth_signature_method" );
        if ( !$signature_method )
        {
            $signature_method = "PLAINTEXT";
        }
        if ( !in_array( $signature_method, array_keys( $this->signature_methods ) ) )
        {
            throw new OAuthException( "Signature method '{$signature_method}' not supported try one of the following: ".implode( ", ", array_keys( $this->signature_methods ) ) );
        }
        return $this->signature_methods[$signature_method];
    }

    private function get_consumer( &$request )
    {
        $consumer_key = @$request->get_parameter( "oauth_consumer_key" );
        if ( !$consumer_key )
        {
            throw new OAuthException( "Invalid consumer key" );
        }
        $consumer = $this->data_store->lookup_consumer( $consumer_key );
        if ( !$consumer )
        {
            throw new OAuthException( "Invalid consumer" );
        }
        return $consumer;
    }

    private function get_token( &$request, $consumer, $token_type = "access" )
    {
        $token_field = @$request->get_parameter( "oauth_token" );
        $token = $this->data_store->lookup_token( $consumer, $token_type, $token_field );
        if ( !$token )
        {
            throw new OAuthException( "Invalid {$token_type} token: {$token_field}" );
        }
        return $token;
    }

    private function check_signature( &$request, $consumer, $token )
    {
        $timestamp = @$request->get_parameter( "oauth_timestamp" );
        $nonce = @$request->get_parameter( "oauth_nonce" );
        $this->check_timestamp( $timestamp );
        $this->check_nonce( $consumer, $token, $nonce, $timestamp );
        $signature_method = $this->get_signature_method( $request );
        $signature = $request->get_parameter( "oauth_signature" );
        $valid_sig = $signature_method->check_signature( $request, $consumer, $token, $signature );
        if ( !$valid_sig )
        {
            throw new OAuthException( "Invalid signature" );
        }
    }

    private function check_timestamp( $timestamp )
    {
        $now = time( );
        if ( $this->timestamp_threshold < $now - $timestamp )
        {
            throw new OAuthException( "Expired timestamp, yours {$timestamp}, ours {$now}" );
        }
    }

    private function check_nonce( $consumer, $token, $nonce, $timestamp )
    {
        $found = $this->data_store->lookup_nonce( $consumer, $token, $nonce, $timestamp );
        if ( $found )
        {
            throw new OAuthException( "Nonce already used: {$nonce}" );
        }
    }

}

class OAuthDataStore
{

    public function lookup_consumer( $consumer_key )
    {
    }

    public function lookup_token( $consumer, $token_type, $token )
    {
    }

    public function lookup_nonce( $consumer, $token, $nonce, $timestamp )
    {
    }

    public function new_request_token( $consumer )
    {
    }

    public function new_access_token( $token, $consumer )
    {
    }

}

class SimpleOAuthDataStore extends OAuthDataStore
{

    private $dbh = NULL;

    public function __construct( $path = "oauth.gdbm" )
    {
        $this->dbh = dba_popen( $path, "c", "gdbm" );
    }

    public function __destruct( )
    {
        dba_close( $this->dbh );
    }

    public function lookup_consumer( $consumer_key )
    {
        $rv = dba_fetch( "consumer_{$consumer_key}", $this->dbh );
        if ( $rv === FALSE )
        {
            return NULL;
        }
        $obj = unserialize( $rv );
        if ( !$obj instanceof OAuthConsumer )
        {
            return NULL;
        }
        return $obj;
    }

    public function lookup_token( $consumer, $token_type, $token )
    {
        $rv = dba_fetch( "{$token_type}_{$token}", $this->dbh );
        if ( $rv === FALSE )
        {
            return NULL;
        }
        $obj = unserialize( $rv );
        if ( !$obj instanceof OAuthToken )
        {
            return NULL;
        }
        return $obj;
    }

    public function lookup_nonce( $consumer, $token, $nonce, $timestamp )
    {
        if ( dba_exists( "nonce_{$nonce}", $this->dbh ) )
        {
            return TRUE;
        }
        dba_insert( "nonce_{$nonce}", "1", $this->dbh );
        return FALSE;
    }

    public function new_token( $consumer, $type = "request" )
    {
        $key = md5( time( ) );
        $secret = time( ) + time( );
        $token = new OAuthToken( $key, md5( md5( $secret ) ) );
        if ( !dba_insert( "{$type}_{$key}", serialize( $token ), $this->dbh ) )
        {
            throw new OAuthException( "doooom!" );
        }
        return $token;
    }

    public function new_request_token( $consumer )
    {
        return $this->new_token( $consumer, "request" );
    }

    public function new_access_token( $token, $consumer )
    {
        $token = $this->new_token( $consumer, "access" );
        dba_delete( "request_".$token->key, $this->dbh );
        return $token;
    }

}

class OAuthUtil
{

    public static function urlencode_rfc3986( $input )
    {
        if ( is_array( $input ) )
        {
            return array_map( array( "OAuthUtil", "urlencode_rfc3986" ), $input );
        }
        if ( is_scalar( $input ) )
        {
            return str_replace( "+", " ", str_replace( "%7E", "~", rawurlencode( $input ) ) );
        }
        return "";
    }

    public static function urldecode_rfc3986( $string )
    {
        return rawurldecode( $string );
    }

}

?>
